/**************************************************************************//**
*   @file   AD5443.c
*   @brief  Implementation of AD5443 Driver for Microblaze processor.
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
#define AD5443_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "AD5443.h"
#include "../drivers/spi.h"
#include "xparameters.h"

#define SPI_BASEADDR XPAR_AXI_SPI_0_BASEADDR

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
unsigned char   txBuffer[4]     = {0,}; // Tx SPI Buffer.
unsigned char   rxBuffer[4]     = {0,}; // Rx SPI Buffer.
static int		prevDacSlider   = 0;
static int      prevResetZero	= 0;
static int		prevResetMid	= 0;

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
volatile int*	AD5443_dacSlider	= &AD5443_Data[0].iData;
volatile char*	AD5443_ResetZero	= &AD5443_Data[1].chData;
volatile char*	AD5443_ResetMid		= &AD5443_Data[2].chData;
volatile char*	AD5443_SwitchEdge	= &AD5443_Data[3].chData;

/**************************************************************************//**
 * @brief Initialize SPI and Initial Values for AD5443 Board.
 *
 * @param None.
 *
 * @return TRUE or FALSE.
******************************************************************************/
int AD5443_Init( void )

{
	int cfgValue = 0;
	char txBuffer[3] = {0, 0, 0};
	
	// setup SPI interface
    SPI_Init(SPI_BASEADDR, 0, 0, 1);
	
	// Select Command Type and DAC, add desired Data
	cfgValue = CMD_DAISY_DIS;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[1] = (cfgValue & 0x0000FF);
	
	// Transfer data via SPI
	SPI_TransferData(SPI_BASEADDR, 2, txBuffer, 0, NULL, 3);
	
	return(TRUE);
}

void AD5443_WriteUpdateDacReg(short rxData)
{
	int cfgValue = 0;
	char txBuffer[3] = {0, 0, 0};
	
	// Select Command Type and DAC, add desired Data
	cfgValue = CMD_LOAD_UP | rxData;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[1] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	SPI_TransferData(SPI_BASEADDR, 2, txBuffer, 0, NULL, 3);
}

void AD5443_Daisy(void)
{
	int cfgValue = 0;
	char txBuffer[3] = {0, 0, 0};
	
	// Select command
	cfgValue = CMD_DAISY_DIS;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[1] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	SPI_TransferData(SPI_BASEADDR, 2, txBuffer, 0, NULL, 3);
}

void AD5443SwitchEdge(void)
{
	int cfgValue = 0;
	char txBuffer[3] = {0, 0, 0};
	
	// Select command
	cfgValue = CMD_RISING_EDGE;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[1] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	SPI_TransferData(SPI_BASEADDR, 2, txBuffer, 0, NULL, 3);
}

void AD5443ResetZero(void)
{
	int cfgValue = 0;
	char txBuffer[3] = {0, 0, 0};
	
	// Select command
	cfgValue = CMD_CLR_DAC_ZERO;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[1] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	SPI_TransferData(SPI_BASEADDR, 2, txBuffer, 0, NULL, 3);
}

void AD5443_ResetMidscale(void)
{
	int cfgValue = 0;
	char txBuffer[3] = {0, 0, 0};
	
	// Select command
	cfgValue = CMD_CLR_DAC_MID;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[1] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	SPI_TransferData(SPI_BASEADDR, 2, txBuffer, 0, NULL, 3);
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return TRUE
******************************************************************************/
int AD5443_UpdateParameters(void)
{

	if(*AD5443_ResetZero != prevResetZero)
	{
		AD5443ResetZero();
		prevResetZero = *AD5443_ResetZero;
	}
	else if(*AD5443_ResetMid != prevResetMid)
	{
		AD5443_ResetMidscale();
		prevResetMid = *AD5443_ResetMid;
	}
	if(*AD5443_dacSlider != prevDacSlider)
	{
		AD5443_WriteUpdateDacReg(*AD5443_dacSlider);
		prevDacSlider = *AD5443_dacSlider;
	}
    return(TRUE);
}
